/**
 * An archive message format that supports easy export.
 * @author Brian Smyth
 */
public class BPSArchiveMessage {
    private String name;
    private String text;
    //private Date date; // This isn't supported yet by the archive format

    /**
     * @param n The message's name
     * @param t The message's text
     */
    public BPSArchiveMessage(String n, String t) {
        name = n;
        text = t;
    }

    /**
     * @return The message's name.
     */
    public String getName() {
        return name;
    }

    /**
     * @return The message's text.
     */
    public String getText() {
        return text;
    }

    /**
     * Add more text to the message.
     * @param txt The additional text.
     * @param sep The separator string.
     */
    public void appendText(String txt, String sep) {
        text += sep + txt;
    }

    /**
     * @return A string representation of the message.
     */
    public String toString() {
        return "Name = " + name + " || Text = " + text;
    }

    /**
     * Get rid of extra spaces in the message text (seems to be caused by
     * something strange bewteen the Java and Delphi versions).
     */
    public void squish() {
        text = text.replaceAll("\\s+", " ");
        text = text.replaceAll("^ ", "");
        text = text.replaceAll(" $", "");
    }
}
