import org.xml.sax.*;
import org.xml.sax.helpers.*;
import java.util.Vector;

/**
 * SAX handler for reading the XML transcripts.
 * @author Brian Smyth
 */
public class BPSMessageHandler extends DefaultHandler {
    private int char_data;
    private final int NAME_DATA = 1;
    private final int TEXT_DATA = 2;
    private String tmp_name;
    private String tmp_msg;
    private Vector messages;

    /**
     * Default constructor.
     */
    public BPSMessageHandler() {
        messages = new Vector();
        tmp_name = "";
        tmp_msg = "";
    }

    /**
     * Receive notification of the start of an element.
     */
    public void startElement(String namespace_uri, String local_name,
            String q_name, Attributes attrs) {
        if (local_name.equals("name")) {
            char_data = NAME_DATA;
        } else if (local_name.equals("text")) {
            char_data = TEXT_DATA;
        }
    }

    /**
     * Receive notification of character data. Beware that it's not
     * defined how the chunks of characters are given by the parser.
     */
    public void characters(char[] chars, int start, int length) {
        String tmp_str = "";
        for (int i = start; i < start + length; i++) {
            tmp_str += chars[i];
        }
        if (char_data == NAME_DATA) {
            tmp_name += tmp_str;
        } else if (char_data == TEXT_DATA) {
            tmp_msg += tmp_str;
        }
    }

    /**
     * Receive notification of the end of an element.
     */
    public void endElement(String namespace_uri, String local_name,
            String q_name) {
        if (local_name.equals("message")) {
            tmp_name = tmp_name.trim();
            tmp_msg = tmp_msg.trim();
            messages.add(new BPSArchiveMessage(tmp_name, tmp_msg));
            tmp_name = "";
            tmp_msg = "";
        }
    }

    /**
     * Return the messages after we've parsed.
     * @return Vector of BPSArchiveMessages.
     */
    public Vector getMessages() {
        return messages;
    }
}
