import java.io.BufferedReader;
import java.io.StringReader;
import java.io.IOException;
import java.util.Vector;
import java.util.Iterator;
import java.util.regex.*;

/**
 * A transcript parser that's designed to interpret and index the plain text
 * format used in the Java and Delphi versions of the Virtual Classroom.
 * @author Brian Smyth
 */
public class BPSTranscriptParser {
    private String transcript;
    private JACIndexer indexr;
    private Vector messages;

    /**
     * @param msg The transcript text.
     */
    public BPSTranscriptParser(String msg) {
        transcript = msg;
        //indexr = new JACIndexer();
        indexr = Global.indexer;
        messages = new Vector();
    }

    /**
     * @return The Vector of BPSArchiveMessages.
     */
    public Vector getMessages() {
        return messages;
    }

    /**
     * Parse the transcript, creating messages Vector.
     */
    public void parseTranscript() {
        // tokenize here and append to messages
        BufferedReader br = new BufferedReader(new StringReader(transcript));
        BPSArchiveMessage tmp_msg = null;
        Pattern attrib_pat = Pattern.compile("^\\[(Audio (from)?)?\\s*(\\w+ \\w+)\\s*(to (\\w+ \\w+) in [Pp]rivate)?\\]$");
        while (true) {
            String line = "";
            try {
                line = br.readLine();
            } catch (IOException e) {
                System.out.println("Problem reading.");
                e.printStackTrace();
            }
            if (line == null) { // no more input, write out the last one
                tmp_msg.squish();
                messages.add(tmp_msg);
                //System.out.println(tmp_msg);
                break;
            }

            Matcher m = attrib_pat.matcher(line);
            if (m.matches()) { // start of a new message
                /*for (int i = 0; i < m.groupCount(); i++) {
                    System.out.println("group(" + i + ") = " + m.group(i));
                }*/
                // write out old one
                if (tmp_msg != null) {
                    tmp_msg.squish();
                    messages.add(tmp_msg);
                    // System.out.println(tmp_msg);
                }

                tmp_msg = new BPSArchiveMessage(m.group(3), "");
            } else { // continuation of a message
                tmp_msg.appendText(line, " ");
            }
        }
    }

    /**
     * Index the messages via the JACIndexer.
     */
    public void indexMessages() {
        Iterator it = messages.iterator();
        while (it.hasNext()) {
            BPSArchiveMessage a_msg = (BPSArchiveMessage) it.next();

            indexr.indexMessage(a_msg);
        }
    }
}
