import java.util.Vector;
import java.util.Iterator;

/**
 * Provides an export mechanism from plain text archive format to XML.
 * @author Brian Smyth
 */
public class BPSXMLTranscriptExporter {
    private BPSTranscriptParser parser;
    private Vector messages;
    

    public BPSXMLTranscriptExporter() {
        parser = null;
        messages = null;
    }

    /**
     * Convert the transcript and return XML.
     * @param xcript The transcript to parse.
     * @return XML-formatted String.
     */
    public String convertTranscript(String xcript) {
        String xml = "";

        parser = new BPSTranscriptParser(xcript);
        parser.parseTranscript();
        messages = parser.getMessages();

        /* We could use something more sophisticated here.
         * But as of now, just writing XML without dealing with
         * DOM etc. is probably faster. It's a pretty flat hierarchy
         * anyway.
         */

        xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<VCTranscript version=\"1.0\">\n";

        Iterator it = messages.iterator();

        while (it.hasNext()) {
            BPSArchiveMessage msg = (BPSArchiveMessage) it.next();
            xml += "\t<message>\n\t\t<name>"
                + msg.getName()
                + "</name>\n\t\t<text>"
                + msg.getText()
                + "</text>\n\t</message>\n";
        }
        
        xml += "</VCTranscript>";
        return xml;
    }
}
