// -------------------------------------------------------------------------
// Name: Christopher Santamaria
// Date:
//--------------------------------------------------------------------------

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.border.*;
import javax.swing.filechooser.*;
import java.io.*;
import javax.swing.text.*;
import javax.swing.text.rtf.*;
import java.io.ByteArrayOutputStream;

public class ChatPanel extends JPanel implements ActionListener, ItemListener, KeyListener
{
   private boolean debug = false;

   // Encapsulate messageTextPane|ChatOptions|inputTextPane
   JPanel chatJPanel = null;

   // Displays messages received
   JScrollPane messageScrollPane = null;
   JTextPane messageTextPane = null;

   // Area to type outgoing messges
   JScrollPane inputScrollPane = null;
   JTextPane inputTextPane = null;

   // For colored text in JTextPanes
   StyledDocument logDoc = null;

   // Encapsulate privateMessage Opt|Audio check|textColor Opt
   JPanel chatOptions = null;

   // Private message
   JLabel privateLabel = null;
   JComboBox privateJComboBox = null;
   String privateArray[] = { "ALL" };

   // Audio
   JCheckBox audio = null;
   boolean useAudio;

   // Text Color
   JLabel textLabel = null;
   JComboBox textColor = null;
   String colors[] = { "Black", "Blue", "Cyan",
                       "Dark Gray", "Gray", "Green",
                       "Light Gray", "Magenta", "Orange",
                       "Pink", "Red", "White", "Yellow" };


   // Encapsulate save|clear|send buttons
   JPanel buttonJPanel = null;

   JButton saveJButton = null;
   JButton clearJButton = null;
   JButton sendJButton = null;

	//JAC
   JButton searchJButton = null;
	
   // Used to hold the etched border used in this class
   Border etched = null;

   // Offical local variable of the private message option - displayed in status menu
   boolean privatePermission = true;

   // Used to enforce the layout policy
   GridBagLayout gridBagLayout = null;
   GridBagConstraints gridBagConst = null;
   Insets inst = null;

   // -------------------------------------------------------------------------
   // Name: addComponent( Component, int, int, int, int )
   // Return: void
   // Purpose: To add c to the gridbaglayout with passed in constraints
   // -------------------------------------------------------------------------
   private void addComponent( Component c, int row, int column, int width, int height )
   {
      // set gridx and gridy
      gridBagConst.gridx = column;
      gridBagConst.gridy = row;

      // set gridwidth and gridheight
      gridBagConst.gridwidth = width;
      gridBagConst.gridheight = height;

      // set constrants
      gridBagLayout.setConstraints( c, gridBagConst );

      // Add component to layout
      add( c );
   }

   // -------------------------------------------------------------------------
   // Name: ChatPanel( String )
   // Return: none (Constructor)
   // Purpose: To initialize the chatpanel. Sets border title to chatTitle.
   // -------------------------------------------------------------------------
   public ChatPanel( String chatTitle, Dimension d )
   {
      etched = BorderFactory.createEtchedBorder();

      // Minimum size of the panel, used to force scrolling when resized
      setMinimumSize( new Dimension( 500, 200 ) );

      // Set up layout for panel
      gridBagLayout = new GridBagLayout();
      gridBagConst = new GridBagConstraints();
      inst = new Insets( 5, 5, 5, 5 );
      gridBagConst.insets = inst;
      setLayout( gridBagLayout );

      // Initialize components in this panel
      initializeChatPanel();
      initializeButtonPanel();

      // Create a border
      setBorder( BorderFactory.createTitledBorder( etched, chatTitle ) );
   } // End ChatPanel( String )

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void initializeButtonPanel()
   {
      // Set up the button panel
      buttonJPanel = new JPanel( new GridLayout( 4, 1, 10 , 10 ) );
      saveJButton = new JButton( "Save" );
      clearJButton = new JButton( "Clear" );
      sendJButton = new JButton( "Send 'ctrl - x'" );
		//JAC
	  searchJButton = new JButton( "Search" );

      // add components to the button panel
      buttonJPanel.add( saveJButton );
      buttonJPanel.add( clearJButton );
		//JAC
	  buttonJPanel.add( searchJButton );
      buttonJPanel.add( sendJButton );

      // Set action listeners for the two buttons on this dialog
      saveJButton.addActionListener( this );
      clearJButton.addActionListener( this );
      sendJButton.addActionListener( this );
		//JAC
      searchJButton.addActionListener( this );
      
      // Add buttonPanel to layout
      addComponent( buttonJPanel, 4, 7, 2, 4 );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void set( Dimension d )
   {
      messageScrollPane.setPreferredSize( d );
      this.revalidate();
      this.repaint();
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void initializeChatPanel()
   {
      // Create recieved message TextPane and ScrollPane
      messageTextPane = new JTextPane();
      messageScrollPane = new JScrollPane( messageTextPane );
      messageScrollPane.setPreferredSize( new Dimension( 400, 300 ) );
      messageScrollPane.setHorizontalScrollBarPolicy( JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
      messageScrollPane.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

      // Create input TextPane and ScrollPane
      inputTextPane = new JTextPane();
      inputScrollPane = new JScrollPane(inputTextPane);
      inputScrollPane.setPreferredSize( new Dimension( 400, 60 ) );
      inputScrollPane.setHorizontalScrollBarPolicy( JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
      inputScrollPane.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

      // To handle ctrl-x send shortcut
      inputTextPane.addKeyListener( this );

      // Initialize all the different styles
      logDoc = inputTextPane.getStyledDocument();
      initializeStyles();

      // Set up the JTextPane
      messageTextPane.setEditable( false );

      // Add messageScrollPane to layout
      addComponent( messageScrollPane, 2, 2, 5, 4 );

      // Place text audio and color buttons
      initializeChatOptions();

      // Add chatOptions Panel to layout
      addComponent( chatOptions, 6, 2, 5, 1 );

      // Add inputScrollPane Panel to layout
      addComponent( inputScrollPane, 7, 2, 5, 3 );
   } // End initializeChatPanel()

   // -------------------------------------------------------------------------
   // Name: initializeStyles()
   // Return: void
   // Purpose: To create all posible documents for the colors used in the app.
   //          This is so that all possible colors can be seen in the recieved
   //          message window.
   // -------------------------------------------------------------------------
   private void initializeStyles()
   {
      Style defaultStyle = StyleContext.getDefaultStyleContext().getStyle( StyleContext.DEFAULT_STYLE );

      Style blackStyle = logDoc.addStyle( "Black", defaultStyle );
      StyleConstants.setForeground( blackStyle, Color.BLACK );

      Style blueStyle = logDoc.addStyle( "Blue", defaultStyle );
      StyleConstants.setForeground( blueStyle, Color.BLUE );

      Style cyanStyle = logDoc.addStyle( "Cyan", defaultStyle );
      StyleConstants.setForeground( cyanStyle, Color.CYAN );

      Style darkgrayStyle = logDoc.addStyle( "Dark Gray", defaultStyle );
      StyleConstants.setForeground( darkgrayStyle, Color.DARK_GRAY );

      Style grayStyle = logDoc.addStyle( "Gray", defaultStyle );
      StyleConstants.setForeground( grayStyle, Color.GRAY );

      Style greenStyle = logDoc.addStyle( "Green",defaultStyle );
      StyleConstants.setForeground( greenStyle, Color.GREEN );

      Style lightGrayStyle = logDoc.addStyle( "Light Gray", defaultStyle );
      StyleConstants.setForeground( lightGrayStyle, Color.LIGHT_GRAY );

      Style magentaStyle = logDoc.addStyle( "Magenta",defaultStyle );
      StyleConstants.setForeground( magentaStyle, Color.MAGENTA );

      Style orangeStyle = logDoc.addStyle( "Orange",defaultStyle );
      StyleConstants.setForeground( orangeStyle, Color.ORANGE );

      Style pinkStyle = logDoc.addStyle( "Pink", defaultStyle );
      StyleConstants.setForeground( pinkStyle, Color.PINK );

      Style redStyle = logDoc.addStyle( "Red", defaultStyle );
      StyleConstants.setForeground( redStyle, Color.RED );

      Style whiteStyle = logDoc.addStyle( "White", defaultStyle );
      StyleConstants.setForeground( whiteStyle, Color.WHITE );

      Style yellowStyle = logDoc.addStyle( "Yellow", defaultStyle );
      StyleConstants.setForeground( yellowStyle, Color.YELLOW );
   }

   // -------------------------------------------------------------------------
   // Name: initializeChatOptions()
   // Return: void
   // Purpose: To initialize the chat options panel.
   // -------------------------------------------------------------------------
   private void initializeChatOptions()
   {
      chatOptions = new JPanel( new FlowLayout() );

      // Create Private Message things
      privateLabel = new JLabel( "To: " );
      privateJComboBox = new JComboBox( privateArray );

      // Create Audio Message things
      audio = new JCheckBox( "Audio", false );
      audio.addItemListener( this );
      useAudio = false;

      // Create Color Message things
      textLabel = new JLabel( "Text Color: " );
      textColor = new JComboBox( new ColorComboBoxModel( colors ) );
      textColor.setMaximumRowCount( colors.length );
      textColor.setRenderer( new LabelComboRenderer() );
      textColor.addActionListener( this );

      // Add everything to the chatOptions panel
      chatOptions.add( privateLabel );
      chatOptions.add( privateJComboBox );
      chatOptions.add( audio );
      chatOptions.add( textLabel );
      chatOptions.add( textColor );

      // Create a border around chatOptions panel
      chatOptions.setBorder( BorderFactory.createTitledBorder( etched ) );
   } // End initializeChatOptions()

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Used to chage the color of all the text in the inputTextPane
   // -------------------------------------------------------------------------
   private void setLogicalDocument( Color color )
   {
      if( Color.BLACK.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Black" ) );
      else if( Color.BLUE.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Blue" ) );
      else if( Color.CYAN.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Cyan" ) );
      else if( Color.DARK_GRAY.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Dark Gray" ) );
      else if( Color.GRAY.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Gray" ) );
      else if( Color.GREEN.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Green" ) );
      else if( Color.LIGHT_GRAY.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Light Gray" ) );
      else if( Color.MAGENTA.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Magenta" ) );
      else if( Color.ORANGE.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Orange" ) );
      else if( Color.PINK.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Pink" ) );
      else if( Color.RED.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Red" ) );
      else if( Color.WHITE.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "White" ) );
      else if( Color.YELLOW.equals( color ) )
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Yellow" ) );
      else
         logDoc.setLogicalStyle( ( inputTextPane.getCaretPosition() - 1 ), logDoc.getStyle( "Black" ) );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void actionPerformed( ActionEvent e )
   {
      // Used when saving the message text
      int returnVal;
      File file = null;
      FileWriter out = null;

      // Holds the file chooser used to save transcripts
      JFileChooser fc = null;

      // Used to hold text from the text areas
      String text = null;

	if( e.getSource() == searchJButton ) {
		System.out.println("Setting search interface visible");
		Global.searchInterface.setVisible(true);
	}


      // JComboBox ( Change inputTextPane Text Color )
      if( e.getSource() == textColor )
      {
         // Display the selected color in the JCombobox label
         textColor.setForeground(
            StaticFunctions.computeColor( ((String)textColor.getSelectedItem()).toString() ) );

         setLogicalDocument( textColor.getForeground() );
         inputTextPane.requestFocus();
      }

      // JButton ( Save messageTextPane text )
      if( e.getSource() == saveJButton )
      {
         fc = new JFileChooser();
             // BPS start
             BPSFileFilter filt = new BPSFileFilter();
         filt.addExtension("txt");
         filt.addExtension("xml");
         filt.setDescription("Plain text and XML transcripts");
         fc.setFileFilter(filt);
             // BPS end
         returnVal = fc.showSaveDialog( null );

         // Save messageTextPane text
         if( returnVal == JFileChooser.APPROVE_OPTION )
         {
            // get selected text
            file = fc.getSelectedFile();

            // open file writer and write out text
            try {
               out = new FileWriter( file );

               // Replace newline message with system newline (diff between windows, mac and unix)
               // BPS start
               if (fc.getSelectedFile().getName().endsWith("xml")) { // do the XML export
                   BPSXMLTranscriptExporter exportr = new BPSXMLTranscriptExporter();
                   text = exportr.convertTranscript(messageTextPane.getText());
               } else { // do the plain text export
                   text = new String( messageTextPane.getText() );
               }
               //text = text.replaceAll( "\n", StaticFunctions.newline );

               // Write out file
               out.write( text );

               // Flush the stream
               out.flush();
               out.close();
               out = null;
            }
            catch( IOException IO ) {

            }
         }
      } // End else if( saveJButton )

      // JButton ( Clear inputTextPane text )
      if (e.getSource() == clearJButton )
      {
         inputTextPane.setText( "" );
      }

      // JButton ( send to V.C. )
      if( e.getSource() == sendJButton )
      {
         // Make sure message is not null
         if( inputTextPane.getText() == "" )
         {
            JOptionPane.showMessageDialog( null,
               "Can not send empty message",
               "Malformed Message",
               JOptionPane.ERROR_MESSAGE );

            return;
         }

         sendPrivateChatMessage();

         // Clear inputTextPane
         inputTextPane.setText( "" );

         // Give inputTextPane focus for another message
         inputTextPane.requestFocus();
      }
   }  // End actionPerformed( ActionEvent )

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void setPrivatePermission( boolean b )
   {
      privatePermission = b;

      // Enable, or disable the Private message option
      setPrivateComboBox();
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public boolean getPrivatePermission()
   {
      return privatePermission;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: If disabling the combobox, disable it, and then set the index
   //          to 'ALL'
   // -------------------------------------------------------------------------
   public void setPrivateComboBox()
   {
      // message is true if DISABLED
      // map true to false (disable) and false to true(enable)
      privateJComboBox.setEnabled( !privatePermission );

      // Force 'ALL' when private chat is disabled
      privateJComboBox.setSelectedIndex( 0 );
   }


   // -------------------------------------------------------------------------
   // Name: fillComoboBox()
   // Return: void
   // Purpose: Whenever the state of the VC changes, messages 4,7 and 9, this
   //          function will update the list of availible people to private msg
   // -------------------------------------------------------------------------
   public void fillComoboBox()
   {
      // Get String array of all people online including "ALL" option
      privateArray = Global.classroomPanel.fillPrivateMessageDropDownList();

      if( debug ) {
         System.out.println( "fillComboBox" );

         for( int i = 0; i < privateArray.length; i++ )
            System.out.println( "i = " + privateArray[i] );
      }

      // Clear the combo box of all current items
      privateJComboBox.removeAllItems();

      // Now add all items back into the combobox. Used to add and delete people
      // as they sign on and off
      for( int i = 0; i < privateArray.length; i++ )
         if( privateArray[i] != null )
            privateJComboBox.addItem( privateArray[i] );

   } // End fillComoboBox()

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Handle the key pressed event from the text field.
   // -------------------------------------------------------------------------
    public void keyPressed(KeyEvent e)
    {
       int keyCode = e.getKeyCode();
       int modifiers = e.getModifiersEx();

       // System.out.println( keyCode + " " + modifiers );

       if( keyCode == 88 && modifiers == 128 )
       {
          ActionEvent ae = new ActionEvent( sendJButton, 3, "hello" );
          actionPerformed( ae );
       }
    }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Handle the key typed event from the text field.
   // -------------------------------------------------------------------------
    public void keyTyped(KeyEvent e) {

    }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Handle the key released event from the text field
   // -------------------------------------------------------------------------
    public void keyReleased(KeyEvent e) {

    }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: To handle Audio check box action
   // -------------------------------------------------------------------------
   public void itemStateChanged( ItemEvent e )
   {
      if( e.getSource() == audio )
      {
         if( useAudio == false )
            useAudio = true;
         else
            useAudio = false;
      }
   }

   // -------------------------------------------------------------------------
   // Name: disableAudioBox()
   // Return: void
   // Purpose: Orginally intended to be used when FreeTTS audio was not
   //  available, unfortunately i could never figure out how to do that because
   //  of ClassNotFound errors by the jvm.
   // -------------------------------------------------------------------------
   public void disableAudioBox()
   {
      audio.setEnabled( false );
   }

   // -------------------------------------------------------------------------
   // Name: getDefaultButton()
   // Return: JBUTTON the send button on the chatPanel
   // Purpose: To return the send button from the Global.chatPanel class
   // -------------------------------------------------------------------------
   public JButton getDefaultButton( )
   {
      return sendJButton;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Handles COMMAND_MESSAGE (2). To handle incoming chat message.
   // -------------------------------------------------------------------------
   public void receiveChatMessage( IncomingMessage msg )
   {
      // Message parsing
      String name = null;
      Color msgColor = null;;
      boolean audioText = false;
      String text = null;

      // Get the name of the person who sent the message
	  System.out.println("In chat panel");
      name =  msg.getName();

      // Map color of text to available color
      msgColor = StaticFunctions.computeColorComp( msg.getNextVariable() );

      // Should we speak?
      if( msg.getNextVariable().compareTo( "-1" ) == 0 )
         audioText = true;
      else
         audioText = false;

      // Get the actual text of the message
      text = msg.getNextVariable();

      // Replace windows newline chars '\c\r' with java standard '\n'
      text = text.replace( StaticFunctions.delphiNewline, '\n' );

      // If we need to speak, now is the time
      if( audioText == true )
      {
         if( Global.speechAgent.isOnline() == true )
            Global.speechAgent.speak( text );
      }

      // If we recieved an audio messgae, change message header
      if( audioText == true )
         text = "[Audio " + name + "]\n" + text + "\n";
      else
         text = "[" + name + "]\n" + text + "\n";

      // Insert text into the document
      insertText( text, msgColor );

   } // End receiveChatMessage( IncomingMessage )

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Handles COMMAND_PRIVATE_MESSAGE (13) To handle incoming chat message.
   // -------------------------------------------------------------------------
   public void receivePrivateChatMessage( IncomingMessage msg )
   {
      // Message parsing
      String myself = null, sender = null, receiver = null;
      Color msgColor = null;
      boolean audioText = false;
      String text = null;

      // who am i?
      myself = Global.myFirstName + " " + Global.myLastName;

      // get the name of the person who sent the message
      sender = msg.getName();

      // get the intended recepient of the message
      receiver = msg.getNextVariable();

      // if i am not the sender, or not the reciever, return
      if( myself.compareToIgnoreCase( sender ) != 0 )
      {
         if( myself.compareToIgnoreCase( receiver ) != 0 )
            return;
      }

      // Map color of text to available color
      msgColor = StaticFunctions.computeColorComp( msg.getNextVariable() );

      // Should we speak?
      if( msg.getNextVariable().compareTo( "-1" ) == 0 )
         audioText = true;
      else
         audioText = false;

      // Get the actual text of the message
      text = msg.getNextVariable();

      // Replace windows newline chars '\c\r' with java standard '\n'
      text = text.replace( StaticFunctions.delphiNewline, '\n' );

      // If we need to speak, now is the time
      if( audioText == true )
      {
         if( Global.speechAgent.isOnline() == true )
            Global.speechAgent.speak( text );

      }

      // If we recieved an audio messgae, change message header
      if( audioText == true )
         text = "[Audio " + msg.getName() + " to " + receiver + " in Private]\n" + text + "\n";
      else
         text = "[" + msg.getName() + " to " + receiver + " in Private]\n" + text + "\n";

      // Insert text into the document
      insertText( text, msgColor );

   } // End receivePrivateChatMessage( IncomingMessage )

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Handles COMMAND_TEACHER_MESSAGE (12)
   // -------------------------------------------------------------------------
   public void receiveTeacherChatMessage( IncomingMessage msg )
   {
      // Message parsing
      String myself = null, sender = null, name = null;
      Color msgColor = null;;
      boolean audioText = false;
      String text = null;

      // who am i?
      myself = Global.myFirstName + " " + Global.myLastName;
      myself = myself.toLowerCase();

      // get the name of the person who sent the message
      sender = msg.getName();

      // if i did not send this teacher message - forgetaboutit!
      if( myself.compareToIgnoreCase( sender ) != 0 )
         return;

      // get the name of the person who sent the message ( me )
      name = msg.getName();

      // Map color of text to available color
      msgColor = StaticFunctions.computeColorComp( msg.getNextVariable() );

      // Should we speak?
      if( msg.getNextVariable().compareTo( "-1" ) == 0 )
         audioText = true;
      else
         audioText = false;

      // Get the actual text of the message
      text = msg.getNextVariable();

      // Replace windows newline chars '\c\r' with java standard '\n'
      text = text.replace( StaticFunctions.delphiNewline, '\n' );

      // If we need to speak, now is the time
      if( audioText == true )
      {
         if( Global.speechAgent.isOnline() == true )
            Global.speechAgent.speak( text );
      }

      // If we recieved an audio messgae, change message header
      if( audioText == true )
         text = "[Audio " + msg.getName() + " to Teacher in Private]\n" + text + "\n";
      else
         text = "[" + msg.getName() + " to Teacher in Private]\n" + text + "\n";

      // Insert text into the document
      insertText( text, msgColor );

   } // receiveTeacherChatMessage( IncomingMessage )


   // -------------------------------------------------------------------------
   // Name: insertText( String, Color )
   // Return: void
   // Purpose: To insert text into the messageTextPane with color msgColor
   // -------------------------------------------------------------------------
   private void insertText( String text, Color msgColor )
   {
      // TextPane document
      StyledDocument doc = null;
      Style style = null;

      doc = (StyledDocument) messageTextPane.getDocument();

      // Create a style object and then set the style attributes
      style = doc.addStyle( "StyleName", null );

      // Set Foreground color
      StyleConstants.setForeground( style, msgColor );

      // Append text to document of messageTextPane
      try {
         doc.insertString( doc.getLength(), text, style );
      }
      catch( BadLocationException badLocE ) {

      }

      // Move down to newly inserted text (wakes up the scroll pane)
      messageTextPane.select( messageTextPane.getText().length(), messageTextPane.getText().length() );
   } // End insertText( String, Color )

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: To send a chat message to the virtual classroom server DEPRECIATED
   // -------------------------------------------------------------------------
   private void sendChatMessage()
   {
      // Create new message
      OutgoingMessage msg = new OutgoingMessage();

      // append variables
      msg.append( Integer.toString( StaticFunctions.COMMAND_MESSAGE ) );

      // Changed for comp with Delphi
      msg.append( Integer.toString( StaticFunctions.computeColorComp( textColor.getForeground() ) ) );

      if( useAudio == true )
         msg.append( "-1" );
      else
         msg.append( "0" );

      String ta = inputTextPane.getText();
      String sa = new String( "" );

      // If text box was empty
      if( sa.compareTo( ta ) == 0 )
      {
         JOptionPane.showMessageDialog( null,
            "Cannot send an empty message",
            "Error",
            JOptionPane.INFORMATION_MESSAGE );

         return;
      }

      ta = ta.replace( '\n' , StaticFunctions.delphiNewline );

      msg.append( ta );

      msg.finalizeChatMessage();

      // Send message to server
      Global.virtualClassroomThread.virtualClassroomSend( msg );
   }

   // -------------------------------------------------------------------------
   // Name: sendPrivateChatMessage()
   // Return: void
   // Purpose: To send a private/teacher/message to the virtual classroom server
   // -------------------------------------------------------------------------
   private void sendPrivateChatMessage()
   {
      // Create new message
      OutgoingMessage msg = new OutgoingMessage();

      String destination = (String)privateJComboBox.getSelectedItem();

      if( destination.compareTo( "ALL" ) == 0 ) {
         // Append the message type
         msg.append( Integer.toString( StaticFunctions.COMMAND_MESSAGE ) );
      }
      else if( destination.indexOf( "Teacher: " ) == 0 ) {
         // Append the message type
         msg.append( Integer.toString( StaticFunctions.COMMAND_MESSAGE_TEACHER ) );
      }
      else {
         // Append the message type
         msg.append( Integer.toString( StaticFunctions.COMMAND_MESSAGE_PRIVATE ) );

         // Append the person intended to recieve the message
         msg.append( destination );
      }

      // Changed for comp with Delphi
      msg.append( Integer.toString( StaticFunctions.computeColorComp( textColor.getForeground() ) ) );

      if( useAudio == true )
         msg.append( "-1" );
      else
         msg.append( "0" );

      String ta = inputTextPane.getText();
      String sa = new String( "" );

      // If text box was empty
      if( sa.compareTo( ta ) == 0 )
      {
         JOptionPane.showMessageDialog( null,
            "Cannot send an empty message",
            "Error",
            JOptionPane.INFORMATION_MESSAGE );

         return;
      }

      // Replace java standard newline with windows newline '\c\r'
      ta = ta.replace( '\n' , StaticFunctions.delphiNewline );

      // Add text to outgoing message
      msg.append( ta );

      // Clean up the message
      msg.finalizeChatMessage();

      // Send message to server
      Global.virtualClassroomThread.virtualClassroomSend( msg );
   } // End sendPrivateChatMessage()

   public String getTranscript()
   {
      return messageTextPane.getText();
   }

}  // End Class ChatPanel
