// -------------------------------------------------------------------------
// Name: Christopher Santamaria
// Date: 
//--------------------------------------------------------------------------

import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.awt.*;

public class ClassroomPanel extends JPanel
{
   boolean debug = false;

   private StudentJLabel teacher = null;
   private StudentJLabel[] array = null;
   private int studentCount;
   private String courseNumber;

   GridBagLayout gridBagLayout = null;
   GridBagConstraints gridBagConst = null;
   Insets inst = null;

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public ClassroomPanel()
   {
      gridBagLayout = new GridBagLayout();
      gridBagConst = new GridBagConstraints();
      inst = new Insets( 5, 5, 5, 5 );

      gridBagConst.insets = inst;

      setLayout( gridBagLayout );
   }


   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Called from NameServer.handleIP( IncomingMessage )
   // -------------------------------------------------------------------------
   public void insertTeacher( String fn, String ln )
   {
      teacher = new StudentJLabel( StaticFunctions.TEACHER, StaticFunctions.OFFLINE, fn, ln );

      addComponent( new JLabel(), 0, 0, 1, 1 );
      addComponent( new JLabel(), 0, 1, 1, 1 );
      addComponent( teacher, 0, 2, 1, 1 );
      addComponent( new JLabel(), 0, 3, 1, 1 );
      addComponent( new JLabel(), 0, 4, 1, 1 );   
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void populateClassroom( IncomingMessage msg )
   {
      String firstName, lastName;
      int state;

      studentCount = ( msg.getMessageSize() - 2 ) / 2;

      array = new StudentJLabel[ studentCount ];

      // Retrieve Students from the msg
      for( int i = 0; i < studentCount; i++  )
      {
         firstName = msg.getNextVariable();
         lastName = msg.getNextVariable();

         array[i] = new StudentJLabel( StaticFunctions.STUDENT, StaticFunctions.OFFLINE, firstName, lastName );
      }

      // add teacher and students to the panel...
      int row = 1, column = 0;

      for( int i = 0; i < studentCount; i++  )
      {
         addComponent( array[i], row, column, 1, 1 );

         column++;

         if( column == 5 )
         { column = 0; row++; }
      }

   }
   
   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   public String[] getRoster() {

      String[] roster = new String[studentCount + 1];
      
      roster[0] = teacher.getName();
      
      for( int i = 0; i < studentCount; i++ )
         roster[i+1] = array[i].getName();

      return roster;
   }
   
   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Used for update (4) and teacher_online messages (7)
   // -------------------------------------------------------------------------
   public void changeStudentState( IncomingMessage msg )
   {
      String studentFirstName, studentLastName;
      int status;   

      studentFirstName = msg.getFirstName();
      studentLastName = msg.getLastName();
      status = Integer.parseInt( msg.getNextVariable() );

      if( debug )
         System.out.println( "Student: " + studentFirstName +  " " + studentLastName + " " + status );

      StudentJLabel student = findStudent( studentFirstName + " " + studentLastName );

      if( debug )
         System.out.println( "Student object: " + student );

      if( student != null )
         student.updateStatus( status );
      else {
         if( debug )
            System.out.println( "changeStudentState failed on " + studentFirstName + " " + studentLastName );
      }
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Used for update (4) and teacher_online messages (7)
   // -------------------------------------------------------------------------
   public void changeTeacherState( IncomingMessage msg )
   {
      int status = Integer.parseInt( msg.getNextVariable() );

      teacher.updateStatus( status );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Used to handle message update list ( 9 )
   // -------------------------------------------------------------------------
   public void updateStudentList( IncomingMessage msg )
   {
      String studentFirstName, studentLastName, status;
      StudentJLabel student = null;

      teacher.updateStatus( Integer.parseInt( msg.getNextVariable() ) );

      while( msg.hasMoreVariables() )
      {
         studentFirstName = msg.getNextVariable();
         studentLastName = msg.getNextVariable();
         status = msg.getNextVariable();

         student = findStudent( studentFirstName + " " + studentLastName );

         if( student != null )
            student.updateStatus( Integer.parseInt( status ) );
         
      }
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void addComponent( Component c, int row, int column, int width, int height )
   {
      // set gridx and gridy
      gridBagConst.gridx = column;
      gridBagConst.gridy = row;

      // set gridwidth and gridheight
      gridBagConst.gridwidth = width;
      gridBagConst.gridheight = height;

      // set constrants
      gridBagLayout.setConstraints( c, gridBagConst );
      add( c );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public StudentJLabel findStudent( String name )
   {
      for( int i = 0; i < studentCount; i++ )
         if( array[i].isMatch( name ) == true )
            return array[i];

      return (StudentJLabel) null;
   }

   // -------------------------------------------------------------------------
   // Name: fillPrivateMessageDropDownList()
   // Return: String[] ( all people online )
   // Purpose: To return an array listing all the people currently online, and
   //          therefore private messagable.
   // Used by Jcombox in chat panel.
   // -------------------------------------------------------------------------
   public String[] fillPrivateMessageDropDownList()
   {
      String list[] = new String[studentCount + 2]; // +2,all and teacher
      int count = 0;

      // Message is not private
      list[count++] = "ALL";

      // Make sure teacher is online - do i have to check for name like below?
      if( teacher.getOnlineStatus() == true )
         list[count++] = ( "Teacher: " + teacher.getName() );

      for( int i = 0; i < studentCount; i++ )
      {
         // Make sure student is online
         if( array[i].getOnlineStatus() == true )
         {
            // if student name equal to my name, don't add, cannot private msg myself
            if( array[i].getName().compareToIgnoreCase( Global.myFirstName + " " + Global.myLastName ) == 0 )
               continue;

            list[count++] = array[i].getName();
         }
      }

      return list;
   } // End String[] fillPrivateMessageDropDownList()

} // End Class ClassroomPanel
