import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;

public class FaceAliveEmoticonPanel extends JPanel implements ActionListener, KeyListener
{
   // Emoticon Type Offline, Online, Sad...
   private String emoticonType;

   // Emoticon State
   private JLabel label = null;
   private JTextField field = null;
   private JButton button = null;
   private JLabel previewLabel = null;

   // -------------------------------------------------------------------------
   // Purpose: Constructor
   // -------------------------------------------------------------------------
   public FaceAliveEmoticonPanel( String et ) {

      // Set layout manager
      setLayout( new FlowLayout( FlowLayout.RIGHT ) );
      
      // Set emoticon type
      emoticonType = et;

      // Initialize panel components
      label = new JLabel( emoticonType + " Emoticon Path: " );

      // Image store path, files that we have pushed out (ie current images)
      String imageStorePath = "images" + File.separator + "ImageStore" +
         File.separator + Global.myFirstName.toLowerCase() + 
         "-" + Global.myLastName.toLowerCase() + "-" +
         Global.className.toLowerCase() + "-" + emoticonType + ".jpg";

      // Default images
      String defaultPath = "images" + File.separator + "DefaultEmoticons" +
         File.separator + emoticonType + ".jpg";

      // Test whether user has personal images in the ImageStore
      File imageFile = new File( imageStorePath );

      if( imageFile.exists() == true ) {
         field = new JTextField( imageStorePath, 30 );
      } else {
         field = new JTextField( defaultPath, 30 );
      }

      button = new JButton( "Browse" );
      previewLabel = new JLabel( new ImageIcon( field.getText() ) );

      // Add components to this panel
      add( label );
      add( field );
      add( button );
      add( previewLabel );
   
      // Add button action listener
      button.addActionListener( this );
      field.addKeyListener( this );
      
      // Refresh this panel
      repaint();
   }

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   private void reloadLabel() {

      FileConverter fc = new FileConverter();
      previewLabel.setIcon( new ImageIcon( fc.readFile( field.getText() ) ) );
      fc = null;
   }   

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   public void actionPerformed( ActionEvent e ) {
      
      if( e.getSource() == button ) {
         JFileChooser fc = new JFileChooser();
         int val = fc.showOpenDialog( null );

         if( val == JFileChooser.APPROVE_OPTION ) {
            loadFile( fc.getSelectedFile() );
         }
         else {
            return;
         }
      }
   }  // End function

   // -------------------------------------------------------------------------
   // Purpose: Handle the key pressed event from the text field.
   // -------------------------------------------------------------------------
   public void keyPressed(KeyEvent e) {
      int keyCode = e.getKeyCode();
      int modifiers = e.getModifiersEx();
      
      if( keyCode == 10 && modifiers == 0 ) {
         loadFile( new File( field.getText() ) );
      }
    }

   // -------------------------------------------------------------------------
   // Purpose: Handle the key typed event from the text field.
   // -------------------------------------------------------------------------
   public void keyTyped(KeyEvent e) {
      // do nothing
   }

   // -------------------------------------------------------------------------
   // Purpose: Handle the key released event from the text field
   // -------------------------------------------------------------------------
   public void keyReleased(KeyEvent e) {
      // do nothing  
   }

   // -------------------------------------------------------------------------
   // Purpose: When user selects a file, load it, and then resize it
   // -------------------------------------------------------------------------
   private void loadFile( File selectedFile ) {
      
      // Make sure the file wasn't deleted
      if( selectedFile.exists() == false ) {
      
         JOptionPane.showMessageDialog(
            null,
            "The file you selected no longer exists",
            "File Error",
            JOptionPane.ERROR_MESSAGE );

         updateImage();
         
         return;
      }
      
      // Make sure we have permission to read the selected file
      if( selectedFile.canRead() == false ) {

         JOptionPane.showMessageDialog(
            null,
            "You do not have read permission for the file you selected",
            "File Error",
            JOptionPane.ERROR_MESSAGE );
         
         return;
      }

      // Convert the image and write it back out to the images directory
      JpgImage jpg = null;

      String writePath = "images" + File.separator + "tempImageStore" + 
         File.separator +
         Global.myFirstName.toLowerCase() + "-" + 
         Global.myLastName.toLowerCase() + "-" + 
         Global.className.toLowerCase() + "-" + 
         emoticonType + ".jpg";     

      // Clear the previous icon image from the Jlabel
      previewLabel.setIcon( (ImageIcon) null );
      
      try {
         // Load the new image
         jpg = new JpgImage( selectedFile.getPath() );

         // Resize the image
         jpg.scaleHeight( 32 );
         jpg.scaleWidth( 32 );

         // Write out the new image
         jpg.sendToFile( writePath );
      }
      catch( IOException io ) {

         JOptionPane.showMessageDialog(
            null,
            "There was an error accessing the file you selected",
            "File Error",
            JOptionPane.ERROR_MESSAGE );
         
         return;
      }
      catch( com.sun.image.codec.jpeg.ImageFormatException ife ) {
         
         JOptionPane.showMessageDialog(
            null,
            "The file you selected is not a JPEG file",
            "File Error",
            JOptionPane.ERROR_MESSAGE );

         return;
      }
   
      // Update text field with new image path
      field.setText( writePath );
      
      // Update JLabel with new preview image
      //previewLabel.setIcon( new ImageIcon( writePath ) );
      reloadLabel();

      // Force a refresh of this panel
      repaint();
   }

   // -------------------------------------------------------------------------
   // Purpose: Refresh the panel, load a new icon
   // -------------------------------------------------------------------------
   public void updateImage() {

      previewLabel.setIcon( (ImageIcon) null );

      // Image store path, files that we have pushed out (ie current images)
      String imageStorePath = "images" + File.separator + "ImageStore" +
         File.separator + Global.myFirstName.toLowerCase() + 
         "-" + Global.myLastName.toLowerCase() + "-" +
         Global.className.toLowerCase() + "-" + emoticonType + ".jpg";

      // Default images
      String defaultPath = "images" + File.separator + "DefaultEmoticons" +
         File.separator + emoticonType + ".jpg";

      // Test whether user has personal images in the ImageStore
      File imageFile = new File( imageStorePath );

      if( imageFile.exists() == true ) {
         field.setText( imageStorePath );
      } else {
         field.setText( defaultPath );
      }

      //reviewLabel.setIcon( new ImageIcon( field.getText() ) );
      reloadLabel();
   }

   // -------------------------------------------------------------------------
   // Purpose: Load the default set of icons
   // -------------------------------------------------------------------------
   public void setToDefaultIcon() {
      previewLabel.setIcon( (ImageIcon) null );

      // Default images
      String defaultPath = "images" + File.separator + "DefaultEmoticons" +
         File.separator + emoticonType + ".jpg";

      field.setText( defaultPath );

      //previewLabel.setIcon( new ImageIcon( field.getText() ) );
      reloadLabel();
   }

   // -------------------------------------------------------------------------
   // Purpose: Return the contents of a text field
   // -------------------------------------------------------------------------
   public String getPath() {
      return field.getText();
   }
}
