import java.io.*;
import java.net.*;
import javax.swing.*;
import java.awt.*;

public class FaceAliveThread extends Thread {

   boolean debug = false;

   // Connection
   InetAddress hostAddress = null;
   int port = 0;
   Socket socket = null;

   // Input
   InputStreamReader isr = null;
   BufferedReader socketIn = null;

   // Output
   PrintWriter socketOut;

   // File operations
   FileConverter fc = null;

   String systemNewline = System.getProperty("line.separator");


   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   public FaceAliveThread( String hostIp, String portStr ) {

      super( "FaceAliveThread" );

      // get Host address
      try {
         hostAddress = InetAddress.getByName( hostIp );
      }
      catch( Exception e ) {
         System.err.println( "Error: FaceAlive IP address for the host ('" + 
            hostIp + "')could not be found" );
      }

      port = Integer.parseInt( portStr );

      if( debug )
         System.out.println( "FaceAlive Server Thread started" );

      try {
         socket = new Socket( hostAddress, port );

         socketIn = new BufferedReader( new InputStreamReader( socket.getInputStream() ) );
         socketOut = new PrintWriter( socket.getOutputStream(), true );
      }
      catch( Exception e ) {
         System.err.println( "Exception: Couldn't connect to FaceAlive Server @ " 
            + hostIp + ":" + port );
      }

      if( debug )
         System.out.println( "VirtualClassroom FaceAlive Thread connected" );

      // Decode image from string
      fc = new FileConverter();
   }

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   public boolean isConnected() {

      if( socket != null ) {
         return socket.isConnected();
      }      

      return false;
   }

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   public void run() {

      if( debug ) {
         System.out.println( "FaceAliveThread Run Function" );
      }

      boolean loop = true;
      String data;
      IncomingMessage messageMsg;

      int messageType;

      try {

         while( loop ) {
            String messageString = socketIn.readLine();

            if( messageString == null )
               break;

            if( debug ) {
               //System.out.println( socket.isConnected() );
               //System.out.println( "message recieved \n" + messageString );
            }

            messageMsg = new IncomingMessage( messageString, StaticFunctions.FACE_ALIVE_SERVER );

            if( debug ) {
               System.out.println( "Message: " + messageMsg.toString() );
            }

            messageType = Integer.parseInt( messageMsg.getNextVariable() );

            switch( messageType ) {
               case StaticFunctions.FACE_SERVER_RECV:
                  handleRecv( messageMsg );
                  break;
               default:
                  break;
            }

         }
      } catch( IOException io ) {
         System.out.println( io );
         close();
      }

   }

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   public void close() {
      try {

         if( debug )
            System.out.println("closing socket" );

         socket.close();
      } catch( IOException io ) {
         System.out.println( io + "\nerror closing socket" ); 
      }
   }

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleRecv( IncomingMessage msg ) {

      // Image store path
      String imageStorePath = "images" + File.separator + 
         "ImageStore" + File.separator;  

      // Get name of person these imcoming images belong to
      String name = msg.getNextVariable();
      name = name.toLowerCase();
      String[] split = name.split( " " );

      boolean isMe = false;

      if( split[0].compareTo( Global.myFirstName.toLowerCase() ) == 0 && 
          split[1].compareTo( Global.myLastName.toLowerCase() ) == 0 ) {
         isMe = true;
      }

      // Get class of this person we are recieving
      String className = msg.getNextVariable();
      className = className.toLowerCase();

      // Get label of person corresponding to the name
      StudentJLabel sLabel = Global.classroomPanel.findStudent( name );

      // Release lock on current icon (set it back to the default)
      sLabel.setToDefaultIcon();
      
      if( isMe == true ) {
         Global.facePanel.setToDefaultIcon();
         Global.studentMenu.setToDefaultIcon();
      }

      if( debug ) {
         JOptionPane.showMessageDialog( null,
            "hold",
            "VC FaceAlive Outgoing Message",
            JOptionPane.ERROR_MESSAGE );
      }

      while( msg.hasMoreVariables() == true ) {

         String emoticonType = msg.getNextVariable();
         String fileData = msg.getNextVariable();
         fileData = fileData.replaceAll( "%@", systemNewline );

         byte[] fileArrayData = fc.decodeString( fileData );

         String fileName = imageStorePath + split[0] + "-" + split[1] + "-" +
            className + "-" + emoticonType + ".jpg";

         File file = new File( fileName );

         if( file.exists() == true ) {
            file.delete();
         }

         fc.writeFile( fileArrayData, fileName );
      }

      if( isMe == true ) {
         Global.facePanel.updateImage();
         Global.studentMenu.updateImage();
      }

      sLabel.updateImage();
   }

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   public void faceAliveSend( OutgoingMessage msg ) {

      // Debug
      if( debug ) {
         System.out.println( "message to send: " + msg.getMessage() );

         JOptionPane.showMessageDialog( null,
            msg.getMessage(),
            "VC FaceAlive Outgoing Message",
            JOptionPane.ERROR_MESSAGE );
      }

      // Send out the message!
      socketOut.println( msg.getMessage() );

      // Increment activity (measure liveness)
      Global.sleepTimer.incrementActivity();

      // If we were sleeping, and now we just sent a message,
      // restore the icon to what it was prior to sleeping
      if( Global.sleepTimer.isSleeping() == true ) {
         Global.sleepTimer.setSleeping( false );
         Global.sleepTimer.restorePreviousStatus();
      }
   
      // Debug
      if( debug )
         System.out.println( "message sent" );
   }

} // End Class FaceAliveThread

/*
   Face Alive Message Specification
   --------------------------------

   PUSH
   ----
   HEADER$$$MSGID$$$FIRST AND LAST NAME$$$CLASS($$$EMOTICON TYPE$$$IMAGE)+

   PULL
   ----
   HEADER$$$MSGID$$$CLASS$$$(FIRST AND LAST NAME$$$EMOTICON TYPE$$$IMAGE HASH)+

   RECV
   ----
   HEADER$$$MSGID$$$FIRST AND LAST NAME$$$CLASS($$$EMOTICON TYPE$$$IMAGE)+
*/
