import javax.swing.*;
import java.util.LinkedList;
import java.util.regex.*;

public class FrequencyRecord
{
	boolean popupErrors = true; // Display errors as JOptionPane?
	
	private String keyword;
	private LinkedList userNames;
	private int frequencyCount;
	
	// -------------------------------------------------------------------------
	// Name:
	// Return:
	// Purpose: Constructor
	// -------------------------------------------------------------------------   
	public FrequencyRecord( String key, boolean val )
	{
		keyword = key;
		userNames = new LinkedList();   
	}
	
	// -------------------------------------------------------------------------
	// Name:
	// Return:
	// Purpose: Constructor
	// -------------------------------------------------------------------------   
	public FrequencyRecord( String record )
	{
		
		record = parseKeyword( record );
		
		userNames = new LinkedList();
		record = parseUserNames( record );
		
		parseFrequency( record );
	}
	
	// -------------------------------------------------------------------------
	// Name:
	// Return: The record without the keyword
	// Purpose: To remove the keyword from the record
	// -------------------------------------------------------------------------    
	private String parseKeyword( String record )
	{
		int index = record.indexOf( ";" );
		
		keyword = record.substring( 0, index );
		
		keyword = keyword.toLowerCase();
		
		record = record.substring( index + 1, record.length() );
		
		return record;
	}
	
	// -------------------------------------------------------------------------
	// Name:
	// Return: The record without usernames
	// Purpose: To remove the usernames from the record
	// -------------------------------------------------------------------------    
	private String parseUserNames( String record )
	{
		int index = record.lastIndexOf( ";" );
		
		String names = record.substring( 0, index );
		
		String array[] = null;
		
		try {
			array = names.split( ";" );
		}
		catch( PatternSyntaxException pse ) {
			
			if( popupErrors ) 
				popup( "Syntax error in FrequencyRecord " + names );
			else
				console( "Syntax error in FrequencyRecord " + names );
			
			System.exit( 0 );
		}
		
		for( int i = 0; i < array.length; i++ )
			userNames.add( array[i] );
		
		record = record.substring( index + 1, record.length() );
		
		return record;
	}
	
	// -------------------------------------------------------------------------
	// Name:
	// Return: void
	// Purpose: To parse the frequency of the keyword from the record
	// -------------------------------------------------------------------------    
	private void parseFrequency( String record )
	{
		try {
			frequencyCount = Integer.parseInt( record );
		}
		catch( NumberFormatException nfe ) {
			
			if( popupErrors ) 
				popup( "NumberFormatException in FrequencyRecord " + record  );
			else
				console( "NumberFormatException in FrequencyRecord " + record );
			
			System.exit( 0 );    
		}
	}
	
	// -------------------------------------------------------------------------
	// Name:
	// Return: boolean
	// Purpose: To test whether key is the keyword of this record
	// ------------------------------------------------------------------------- 
	public boolean isKeyword( String key )
	{
		if( keyword.compareToIgnoreCase( key ) == 0 )
			return true;
		
		return false;
	}
	
	// -------------------------------------------------------------------------
	// Name:
	// Return: boolean
	// Purpose: To test whether name has been indexed to this keyword
	// -------------------------------------------------------------------------    
	public boolean isUser( String name )
	{
		for( int i = 0; i < userNames.size(); i++ )
			if( ((String)userNames.get( i )).compareToIgnoreCase( name ) == 0 )
				return true;   
		
		return false;            
	}
	
	// -------------------------------------------------------------------------
	// Name: addUser( String )
	// Return: void
	// Purpose: To add new user to this freq record
	// -------------------------------------------------------------------------    
	public void addUser( String name )
	{
		userNames.add( name );
	}   
	
	// -------------------------------------------------------------------------
	// Name: incCount()
	// Return: void
	// Purpose: Increment the frequency this keyword
	// -------------------------------------------------------------------------    
	public void incCount()
	{
		frequencyCount++;
	}
	
	// -------------------------------------------------------------------------
	// Name:
	// Return:
	// Purpose:
	// -------------------------------------------------------------------------   
	public String toString()
	{
		String output = new String();
		
		output = keyword + ";";
		
		for( int i = 0; i < userNames.size(); i++ )
			output = output.concat( (String)userNames.get( i ) + ";" );
		
		output = output.concat( (new Integer( frequencyCount )).toString() );   
		
		return output;
	}
	
	// -------------------------------------------------------------------------
	// Name:
	// Return: 
	// Purpose:
	// -------------------------------------------------------------------------     
	private void popup( String error )
	{
		JOptionPane.showMessageDialog( null, error,
									   "FrequencyRecord Class Error", 
									   JOptionPane.ERROR_MESSAGE );
	}
	
	// -------------------------------------------------------------------------
	// Name:
	// Return: 
	// Purpose:
	// -------------------------------------------------------------------------     
	private void console( String error )
	{
		System.err.println( error );
	}
	
} // End class FrequencyRecord
