import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;


import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.ParallelMultiSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Hits;
import org.apache.lucene.queryParser.QueryParser;

public class JACSearchInterface extends JFrame implements ActionListener {

	JButton searchButton;
	JTextField queryField;
	JTable resultsTable;
	AbstractTableModel dataModel;
	
	Vector people;
	Analyzer analyzer;
		
	public JACSearchInterface() {
		super("Search Interface");
		analyzer = new StandardAnalyzer();

		people = new Vector();

		setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		
		queryField = new JTextField(50);
		searchButton = new JButton("Search");
		searchButton.addActionListener(this);
	
		dataModel = new AbstractTableModel () {
			public int getRowCount() {
				return people.size();
			}

		    public int getColumnCount() {
				return 1;
	    	}

	    	public Object getValueAt(int row, int col) {
				return people.elementAt(row);
	    	}
		};
	
		resultsTable = new JTable(dataModel);
		JScrollPane scrollPane = new JScrollPane(resultsTable);
	
		JPanel searchPanel = new JPanel(new BorderLayout());
		searchPanel.add( new JLabel("Query: "), BorderLayout.WEST);
		searchPanel.add( queryField, BorderLayout.CENTER);
		searchPanel.add( searchButton, BorderLayout.EAST );

		getContentPane().add(searchPanel, BorderLayout.NORTH);
		getContentPane().add(scrollPane, BorderLayout.CENTER);
		
		pack();
	}

	public void actionPerformed(ActionEvent event) {
		if( event.getSource() == searchButton ) {
			String searchString = queryField.getText();
			people = search(searchString);
			dataModel.fireTableChanged(new TableModelEvent(dataModel));
		}
	}

	public Vector search(String searchString) {
		System.out.println("Searching for: " + searchString);
		Vector matches = new Vector();
	
		System.out.println("\tExamining RelationalIndex");
		String[] terms = searchString.split(" ");
		for(int i = 0; i < terms.length; i++) {
			Vector partialMatches = searchRelationalIndex(terms[i]);
			System.out.println("Matches for term: " + terms[i] + ": " + partialMatches);
			for(int j = 0; j < partialMatches.size(); j++) {
				String match = (String) partialMatches.elementAt(j);
				if(!matches.contains(match)) {
					matches.add(match);
				}
			}
		}
		if(matches.size() == 0) {
			System.out.println("\tExamining Transcripts");
			matches = searchTranscripts(searchString);
		}
		return matches;
	}

	public Vector searchRelationalIndex(String searchTerm) {
		JACRelationalIndex index = new JACRelationalIndex("SessionDB/RelationalIndex");
		return index.search(searchTerm);
	}

	public Vector searchTranscripts(String searchString) {
		Vector results = new Vector();

		Query query;
		try {
			query = QueryParser.parse(searchString, "text", analyzer);
		}
		catch (org.apache.lucene.queryParser.ParseException e) {
			return new Vector();
		}
			
		ParallelMultiSearcher searcher = buildMultiSearcher();

		try {
			Hits hits = searcher.search(query);
			searcher.close();
				
			Hashtable counts = new Hashtable();
				
			for (int i = 0; i < hits.length(); i++) {
				Document doc = hits.doc(i);
				String sender = doc.get("sender");
				String message = doc.get("text");
					
				if(!counts.containsKey(sender)) {
					counts.put(sender, new Integer(0));
				}
				int count = ((Integer) counts.get(sender)).intValue();
				count++;
				counts.put(sender, new Integer(count));
			}
				
			people = new Vector();
			for(Enumeration e = counts.keys(); e.hasMoreElements(); ) {
				String person = (String) e.nextElement();
				int count = ((Integer) counts.get(person)).intValue();
				
				int i = 0;
				for(i = 0; i < people.size(); i++) {
					String person2 = (String) people.elementAt(i);
					int count2 = ((Integer) counts.get(person2)).intValue();
					if( count >= count2 ) {
						break;
					}					
				}
				results.add(i, person);				
			}
		}
		catch(IOException e) {
			System.out.println("Could not perform query");
			return new Vector();
		}
		
		return results;
	}
	
	public ParallelMultiSearcher buildMultiSearcher() {
		Searcher[] searchers = null;

		try {
			File indexDirectory = new File("SessionDB/indexes");
			File[] indexes = indexDirectory.listFiles();
			searchers = new Searcher[indexes.length];

			for(int i = 0; i < indexes.length; i++) {
				searchers[i] = new IndexSearcher(indexes[i].getAbsolutePath().toString());
			}

			return new ParallelMultiSearcher(searchers);
		}
		catch (Exception e) {
			System.out.println(" caught a " + e.getClass() + "\n with message: " + e.getMessage());
			System.exit(1);
		}
		return null;
	}

}