import javax.swing.*;
import java.util.LinkedList;
import java.io.*;
import java.util.regex.*;

public class Search
{
   boolean popupErrors = true; // Display errors as JOptionPane?

   private KeywordList keywordFile = null;
   private FrequencyList freqFile = null;
   private ChatFile chatFile = null;

   // -------------------------------------------------------------------------
   // Name: search( KeywordList, FrequencyList, ChatFile )
   // Return: N/A
   // Purpose: Constructor
   // -------------------------------------------------------------------------   
   public Search( KeywordList key, FrequencyList freq, ChatFile chat )
   {
      keywordFile = key;
      freqFile = freq;
      chatFile = chat;
   }
   
   // -------------------------------------------------------------------------
   // Name: sep( String )
   // Return: String[] 0 - fn ln 1 - words in chat
   // Purpose: To seperate a chat transaction per username saying
   // -------------------------------------------------------------------------      
   public String[] sep( String input ) 
   {
      String array[] = new String[2];
      
      int index = input.indexOf( "]" );

      // Error Handling      
      if( index < 0 ) {

         if( popupErrors ) 
            popup( "Error while parsing user chat response: " + input );
         else
            console( "Error while parsing user chat response: " + input );

         System.exit( 0 );
      }

      array[0] = new String( input.substring( 1, index ) );
      
      String spaceList[] = new String[1];
      
      try {
         spaceList = array[0].split( " " );
      }
      catch( PatternSyntaxException pse ) {

         if( popupErrors ) 
            popup( "Error while spliting user chat response: " + array[0] );
         else
            console( "Error while spliting user chat response: " + array[0] );

         System.exit( 0 );
      }
      
      System.out.println( "*" + array[0] + "*" );
      
      // Regular
      if( spaceList.length == 2 )
         array[0] = spaceList[0] + " " + spaceList[1];
      // Audio
      if( spaceList.length == 3 )
         array[0] = spaceList[1] + " " + spaceList[2];
      // Private
      else if( ( spaceList.length == 7 ) && ( spaceList[0].compareToIgnoreCase( "Audio" ) != 0 ) )
         array[0] = spaceList[0] + " " + spaceList[1];
      // Private Audio
      else if( spaceList.length == 8 )
         array[0] = spaceList[1] + " " + spaceList[2];
      // Teacher
      else if( spaceList.length == 6 )
         array[0] = spaceList[0] + " " + spaceList[1];
      // Teacher Audio
      else if( spaceList.length == 7 )
         array[0] = spaceList[1] + " " + spaceList[2];      

      array[1] = new String( input.substring( index + 1, input.length() ) );
      
      return array;  
   } // End seperate( String )

   // -------------------------------------------------------------------------
   // Name: doWork2()
   // Return: void
   // Purpose: To check each word spoken in the chat transcript against the
   //          keyword list and build relational index
   // -------------------------------------------------------------------------      
   public void doWork2()
   {
      String username = null, text = null;
      int index;
      String array[];

      // For each chat response   
      for( int i = 0; i < chatFile.getResponseNum(); i++ ) {
       
         text = chatFile.getNextReponse( i );
              
         array = sep( text );
       
         username = array[0];
         text = array[1];
         
         array = null;
         
         // parse actual text
         try {
            text = text.replaceAll( "\\p{Punct}", " " );
            text = text.replaceAll( " +", " " );
         }
         catch( PatternSyntaxException pse ) {

            if( popupErrors ) 
               popup( "Pattern Syntax Exception " + text );
            else
               console( "Pattern Syntax Exception " + text );

            System.exit( 0 );

         }

         String keyword;
         LinkedList keywordList;
         
         keywordList = keywordFile.isKeywordInText( text );
         
         // search each word in the transcript
         for( int z = 0; z < keywordList.size(); z++ ) {
            keyword = (String)keywordList.get( z );
            
            if( freqFile.isKeywordARecord( keyword ) ) {
            
               if( freqFile.isUserInRecord( keyword, username ) )
                  freqFile.incFreq( keyword );
               else {
                  freqFile.incFreq( keyword );
                  freqFile.addUser( keyword, username );
               }
            }      
            else {
               freqFile.addNewKeyword( keyword );
               freqFile.incFreq( keyword );
               freqFile.addUser( keyword, username );
            } // else

         }   // End inner for loop

      } // End outer for loop

   } // End doWork()


   // -------------------------------------------------------------------------
   // Name:
   // Return: 
   // Purpose:
   // -------------------------------------------------------------------------     
   private void popup( String error )
   {
      JOptionPane.showMessageDialog( null, error,
                                           "Search Class Error", 
                                           JOptionPane.ERROR_MESSAGE );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return: 
   // Purpose:
   // -------------------------------------------------------------------------     
   private void console( String error )
   {
      System.err.println( error );
   }

} // End Search Class
