// -------------------------------------------------------------------------
// Name: Christopher Santamaria
// Date: 
//--------------------------------------------------------------------------

import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.awt.geom.*;

public class WhiteBoardPanel extends JPanel
{
   private WhiteBoardShape[] array;

   private LinkedList arrayList;

   private float dashes[] = { 10 };
   private BasicStroke dashedLine = new BasicStroke( 2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 8, dashes, 0 );

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public WhiteBoardPanel()
   {
      array = new WhiteBoardShape[0];

      arrayList = new LinkedList();

      setPreferredSize( new Dimension( 600, 400 ) );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void paint(Graphics g)
   {
      g.setColor( Color.WHITE );
      g.fillRect( 0,0, 600, 450 );

      Graphics2D g2d = (Graphics2D) g;
      WhiteBoardShape shape = null;

      for( int i = 0; i < arrayList.size(); i++ )
      {
         shape = (WhiteBoardShape) arrayList.get(i);

         // line
         if( shape.shapeType == StaticFunctions.LINE )
         {
            drawLine( g2d, shape );
         }

         // rectangle
         if( shape.shapeType == StaticFunctions.RECTANGLE )
         {
             drawRectangle( g2d, shape );
         }

         // ellipse
         if( shape.shapeType == StaticFunctions.ELLIPSE )
         {
            drawEllipse( g2d, shape );     
         }

         // circle
         if( shape.shapeType == StaticFunctions.CIRCLE )
         {
            drawCircle( g2d, shape );         
         }
 
         // text
         if( shape.shapeType == StaticFunctions.TEXT )
         {
            g.setColor( Color.BLACK );
            g.drawString( shape.textStr, shape.posX, shape.posY );
         }
      }
   }   

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void addShape( IncomingMessage msg )
   {
      int shapeType;

      arrayList.clear();

      while( msg.hasMoreVariables() )
      {
         shapeType = Integer.parseInt( msg.getNextVariable() );

         System.out.println( "addShape() : shapeType : " + shapeType );

         if( shapeType < 4 )
         {
            arrayList.add( new WhiteBoardShape( shapeType,
                                            Integer.parseInt( msg.getNextVariable() ),
                                            Integer.parseInt( msg.getNextVariable() ),
                                            Integer.parseInt( msg.getNextVariable() ),
                                            Integer.parseInt( msg.getNextVariable() ),
                                            Integer.parseInt( msg.getNextVariable() ),
                                            Integer.parseInt( msg.getNextVariable() ),
                                            Integer.parseInt( msg.getNextVariable() ),
                                            Integer.parseInt( msg.getNextVariable() ),
                                            Integer.parseInt( msg.getNextVariable() ) ) );
         }
 
         if( shapeType == 4 )
         {
            arrayList.add( new WhiteBoardShape( shapeType,
                                            Integer.parseInt( msg.getNextVariable() ),
                                            Integer.parseInt( msg.getNextVariable() ),
                                            msg.getNextVariable() ) );
         }    
      } // End While loop

      for( int i = 0; i < arrayList.size(); i++ )
      {
         System.out.println( " --------------------- " );
         System.out.println( ( (WhiteBoardShape) arrayList.get(i) ).toString() ); 
      }

      repaint(); invalidate();
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void drawLine( Graphics2D g2d, WhiteBoardShape shape )
   {
      g2d.setPaint( StaticFunctions.computeColor( shape.lineColor ) );

      if( shape.lineStyle == StaticFunctions.SOLID )
         g2d.setStroke( new BasicStroke( shape.lineWidth ) );
      else if( shape.lineStyle == StaticFunctions.DOTTED )
         g2d.setStroke( new BasicStroke( shape.lineWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 8, dashes, 0 ) );

      g2d.draw( new Line2D.Double( shape.leftTopX, shape.leftTopY, 
                                   shape.rightBottomX, shape.rightBottomY ) );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void drawRectangle( Graphics2D g2d, WhiteBoardShape shape )
   {
      int width = shape.rightBottomX - shape.leftTopX;
      int height = shape.rightBottomY - shape.leftTopY;
      Rectangle2D.Double rectangle = new Rectangle2D.Double( shape.leftTopX , shape.leftTopY, width, height );

      g2d.setPaint( StaticFunctions.computeColor( shape.lineColor ) ); 

      if( shape.lineStyle == StaticFunctions.SOLID )
      {
         g2d.setStroke( new BasicStroke( shape.lineWidth ) );     
      }
      else if( shape.lineStyle == StaticFunctions.DOTTED )
      {
         g2d.setStroke( new BasicStroke( shape.lineWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 8, dashes, 0 ) );
      }

      g2d.draw( rectangle );

      g2d.setPaint( StaticFunctions.computeColor( shape.brushColor ) );

      rectangle = new Rectangle2D.Double( shape.leftTopX + 1 , shape.leftTopY + 1, width - 1, height - 1 );

      if( shape.brushStyle == StaticFunctions.SOLID )
      {
         g2d.fill( rectangle );
      }
      else if( shape.brushStyle == StaticFunctions.CLEAR )
      {
         g2d.draw( rectangle );
      }  
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void drawEllipse( Graphics2D g2d, WhiteBoardShape shape )
   {
      int width = shape.rightBottomX - shape.leftTopX;
      int height = shape.rightBottomY - shape.leftTopY;
      Ellipse2D.Double ellipse = new Ellipse2D.Double( shape.leftTopX , shape.leftTopY, width, height );

      g2d.setPaint( StaticFunctions.computeColor( shape.lineColor ) ); 

      if( shape.lineStyle == StaticFunctions.SOLID )
      {
         g2d.setStroke( new BasicStroke( shape.lineWidth ) );     
      }
      else if( shape.lineStyle == StaticFunctions.DOTTED )
      {
         g2d.setStroke( new BasicStroke( shape.lineWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 8, dashes, 0 ) );
      }

      g2d.draw( ellipse );

      g2d.setPaint( StaticFunctions.computeColor( shape.brushColor ) );

      ellipse = new Ellipse2D.Double( shape.leftTopX + 1 , shape.leftTopY + 1, width - 1, height - 1 );

      if( shape.brushStyle == StaticFunctions.SOLID )
      {
         g2d.fill( ellipse );
      }
      else if( shape.brushStyle == StaticFunctions.CLEAR )
      {
         g2d.draw( ellipse );
      }    
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void drawCircle( Graphics2D g2d, WhiteBoardShape shape )
   {
      int width = shape.rightBottomX - shape.leftTopX;
      int height = shape.rightBottomY - shape.leftTopY;
      RoundRectangle2D.Double roundRec = new RoundRectangle2D.Double( shape.leftTopX , shape.leftTopY, width, height, 45, 45 );

      g2d.setPaint( StaticFunctions.computeColor( shape.lineColor ) ); 

      if( shape.lineStyle == StaticFunctions.SOLID )
      {
         g2d.setStroke( new BasicStroke( shape.lineWidth ) );     
      }
      else if( shape.lineStyle == StaticFunctions.DOTTED )
      {
         g2d.setStroke( new BasicStroke( shape.lineWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 8, dashes, 0 ) );
      }

      g2d.draw( roundRec );

      g2d.setPaint( StaticFunctions.computeColor( shape.brushColor ) );

      roundRec = new RoundRectangle2D.Double( shape.leftTopX + 1 , shape.leftTopY + 1, width - 2, height - 2, 45, 45 );

      if( shape.brushStyle == StaticFunctions.SOLID )
      {
         g2d.fill( roundRec );
      }
      else if( shape.brushStyle == StaticFunctions.CLEAR )
      {
         g2d.draw( roundRec );
      }  
   }

} // End Class WhiteBoardPanel
